/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.softfloat;

/**
 * softfloat from project http://www.jhauser.us/arithmetic/SoftFloat.html
 * 
 * @author <a href="mailto:protsenko@ispras.ru">Alexander Protsenko</a>
 */

public class CommonNaNT {
  boolean sign; // flag
  long high; // bits64
  long low; // bits64

  /**
   * Constructs an CommonNaNT object.
   */
  public CommonNaNT() {}

  /**
   * Constructs an CommonNaNT object with the current parameters.
   * 
   * @param sign.
   * @param high high part of CommonNaNT.
   * @param low low part of CommonNaNT.
   * 
   * @throws IllegalArgumentException if the argument is {@code null}.
   */
  public CommonNaNT(final boolean sign, final long high, final long low) {
    this.sign = sign;
    this.high = high;
    this.low = low;
  }

  @Override
  public boolean equals(Object obj) {
    if (this == obj) {
      return true;
    }

    if (obj == null) {
      return false;
    }

    if (getClass() != obj.getClass()) {
      return false;
    } else {
      if (this.high == ((CommonNaNT) obj).high && this.low == ((CommonNaNT) obj).low
          && this.sign == ((CommonNaNT) obj).sign) {
        return true;
      }
    }

    return false;
  }

  @Override
  public String toString() {
    return "0x" + (sign ? "1" : "0") + "." + Long.toHexString(high)
                                     + "." + Long.toHexString(low);
  }

  @Override
  public int hashCode() {
    final int prime = 0xffff;
    int result = 1;
    result = (int) (prime * this.low + this.low);
    result = (int) (this.high * result + this.high);
    return result;
  }
}
