package ru.ispras.softfloat;

import java.util.ArrayList;
import java.util.Random;

import org.junit.Test;

/**
 * Tests for jsoftfloat 16fp
 * 
 * @author <a href="mailto:protsenko@ispras.ru">Alexander Protsenko</a>
 */
public class Float16FunctionsTestCase {

  private static final int ARRAY_RANDOM_SIZE = 50;

  private static final boolean PRINT_TESTS_NUMBER = true;
  //private static final boolean RUN_LONGTIME_TESTS = false;

  private static final Random rand = new Random();

  private static final ArrayList<Double> doubleArray = new ArrayList<Double>();
  private static final ArrayList<Float> floatArray = new ArrayList<Float>();

  private static void initTestArray() {
    // Init array of double values
    doubleArray.add(Double.NaN);
    doubleArray.add(Double.NEGATIVE_INFINITY);
    doubleArray.add(Double.POSITIVE_INFINITY);
    doubleArray.add(Double.MAX_VALUE);
    doubleArray.add(Double.MIN_VALUE);
    doubleArray.add(Double.MIN_NORMAL);
    doubleArray.add(Double.longBitsToDouble(-1));
    doubleArray.add(Double.longBitsToDouble(0));
    doubleArray.add(Double.longBitsToDouble(1));
    doubleArray.add(Double.longBitsToDouble(0x7facb75a00000000L)); // NaN
    doubleArray.add(Double.longBitsToDouble(0x7fe0000000000000L));

    for (int i = 0; i < ARRAY_RANDOM_SIZE; i++)
      doubleArray.add(Double.longBitsToDouble(rand.nextLong()));

    doubleArray.add(-5.0);
    doubleArray.add(125.7);
    doubleArray.add(234.23);
    doubleArray.add(44.51);
    doubleArray.add(-215.005);
    doubleArray.add(7.5);
    doubleArray.add(125.6875);

    // Init array of float values
    floatArray.add(Float.NaN);
    floatArray.add(Float.NEGATIVE_INFINITY);
    floatArray.add(Float.POSITIVE_INFINITY);
    floatArray.add(Float.MAX_VALUE);
    floatArray.add(Float.MIN_VALUE);
    floatArray.add(Float.MIN_NORMAL);
    floatArray.add(Float.intBitsToFloat(-1));
    floatArray.add(Float.intBitsToFloat(0));
    floatArray.add(Float.intBitsToFloat(1));
    floatArray.add(Float.intBitsToFloat(0x7facb75a)); // NaN
    floatArray.add(Float.intBitsToFloat(0xff000000));
    for (int i = 0; i < ARRAY_RANDOM_SIZE; i++)
      floatArray.add(Float.intBitsToFloat(rand.nextInt()));

    floatArray.add((float)-5.0);
    floatArray.add((float)125.7);
    floatArray.add((float)234.23);
    floatArray.add((float)44.51);
    floatArray.add((float)-215.005);
    floatArray.add((float)7.5);
    floatArray.add((float)125.6875);
  }

  private static double getDoubleValue(final int i) {
    return doubleArray.get(i);
  }

  @SuppressWarnings("unused")
  private static long getLongValue(final int i) {
    return Double.doubleToRawLongBits(doubleArray.get(i));
  }

  private static float getFloatValue(final int i) {
    return floatArray.get(i);
  }

  @SuppressWarnings("unused")
  private static int getIntValue(final int i) {
    return Float.floatToRawIntBits(floatArray.get(i));
  }

  @SuppressWarnings("unused")
  private static short getShortValue(final int i) {
    return (short) (0xffff & Float.floatToRawIntBits(floatArray.get(i)));
  }

  private static void float16ToFloat64Test(final double inputValue) {
    System.out.print("float16ToFloat64Test InputValue = " + inputValue);

    Float16 middle = Float16Functions.f64_to_f16(inputValue);
    double output = Float16Functions.f16_to_f64(middle);

    System.out.println(" In vs Out: " + inputValue + " == " + output + ", Hex input "
        + Long.toHexString(Double.doubleToRawLongBits(inputValue)) + ", output "
        + Long.toHexString(Double.doubleToRawLongBits(output)));
  }

  private static void float16ToFloat32Test(final float inputValue) {
    System.out.print("float16ToFloat32Test InputValue = " + inputValue);

    Float16 middle = Float16Functions.f32_to_f16(inputValue);
    float output = Float16Functions.f16_to_f32(middle);

    System.out.println(" In vs Out: " + inputValue + " == " + output + ", Hex input "
        + Integer.toHexString(Float.floatToRawIntBits(inputValue)) + ", output "
        + Integer.toHexString(Float.floatToRawIntBits(output)));
  }

  @Test
  public void runTest() {
    System.out.println("Java SoftFloat Tests: Float16.");
    initTestArray();

    for (int i = 0; i < doubleArray.size(); i++) {
      // if (i == 0) break; // TODO

      if (PRINT_TESTS_NUMBER) {
        System.out.println("Conversion test = " + i + " ");
      }
      // Test: Conversion test
      float16ToFloat64Test(getDoubleValue(i));
    }

    for (int i = 0; i < floatArray.size(); i++) {
      // if (i == 0) break; // TODO

      if (PRINT_TESTS_NUMBER)
        System.out.println("Conversion test = " + i + " ");
      // Test: Conversion test
      float16ToFloat32Test(getFloatValue(i));
    }
  }
}
